@@ -25,6 +25,7 @@
 
 #include "arg.h"
 #include "object.h"
+#include "foreign.h"
 #include "boxed.h"
 #include "union.h"
 #include "value.h"
@@ -44,7 +45,7 @@ _gjs_flags_value_is_valid(JSContext   *context,
     /* FIXME: Do proper value check for flags with GType's */
     if (gtype == G_TYPE_NONE)
         return JS_TRUE;
-    
+
     klass = g_type_class_ref(gtype);
 
     /* check all bits are defined for flags.. not necessarily desired */
@@ -753,8 +754,13 @@ gjs_value_to_g_argument(JSContext      *context,
             interface_type = g_base_info_get_type(interface_info);
 
             switch(interface_type) {
-
             case GI_INFO_TYPE_STRUCT:
+                if (g_struct_info_is_foreign((GIStructInfo*)interface_info)) {
+                    return gjs_struct_foreign_convert_to_g_argument(
+                            context, value, type_info, arg_name,
+                            arg_type, transfer, may_be_null, arg);
+                }
+                /* fall through */
             case GI_INFO_TYPE_ENUM:
             case GI_INFO_TYPE_OBJECT:
             case GI_INFO_TYPE_INTERFACE:
@@ -765,7 +771,6 @@ gjs_value_to_g_argument(JSContext      *context,
                 gtype = g_registered_type_info_get_g_type
                     ((GIRegisteredTypeInfo*)interface_info);
                 break;
-
             case GI_INFO_TYPE_VALUE:
                 /* Special case for GValues */
                 gtype = G_TYPE_VALUE;
@@ -1113,6 +1118,141 @@ gjs_value_to_g_argument(JSContext      *context,
     }
 }
 
+/* If a callback function with a return value throws, we still have
+ * to return something to C. This function defines what that something
+ * is. It basically boils down to memset(arg, 0, sizeof(*arg)), but
+ * gives as a bit more future flexibility and also will work if
+ * libffi passes us a buffer that only has room for the appropriate
+ * branch of GArgument. (Currently it appears that the return buffer
+ * has a fixed size large enough for the union of all types.)
+ */
+void
+gjs_g_argument_init_default(JSContext      *context,
+                            GITypeInfo     *type_info,
+                            GArgument      *arg)
+{
+    GITypeTag type_tag;
+
+    type_tag = g_type_info_get_tag( (GITypeInfo*) type_info);
+    if (type_tag != GI_TYPE_TAG_TIME_T) // we handle time_t as a non-int type
+        type_tag = normalize_int_types(type_tag);
+
+    switch (type_tag) {
+    case GI_TYPE_TAG_VOID:
+        arg->v_pointer = NULL; /* just so it isn't uninitialized */
+        break;
+
+    case GI_TYPE_TAG_INT8:
+        arg->v_int8 = 0;
+        break;
+
+    case GI_TYPE_TAG_UINT8:
+        arg->v_uint8 = 0;
+        break;
+
+    case GI_TYPE_TAG_INT16:
+        arg->v_int16 = 0;
+        break;
+
+    case GI_TYPE_TAG_UINT16:
+        arg->v_uint16 = 0;
+        break;
+
+    case GI_TYPE_TAG_INT32:
+        arg->v_int = 0;
+        break;
+
+    case GI_TYPE_TAG_UINT32:
+        arg->v_uint32 = 0;
+        break;
+
+    case GI_TYPE_TAG_INT64:
+        arg->v_int64 = 0;
+        break;
+
+    case GI_TYPE_TAG_UINT64:
+        arg->v_uint64 = 0;
+
+    case GI_TYPE_TAG_TIME_T:
+        arg->v_ulong = 0;
+        break;
+
+    case GI_TYPE_TAG_BOOLEAN:
+        arg->v_boolean = FALSE;
+        break;
+
+    case GI_TYPE_TAG_FLOAT:
+        arg->v_float = 0.0f;
+        break;
+
+    case GI_TYPE_TAG_DOUBLE:
+        arg->v_double = 0.0;
+        break;
+
+    case GI_TYPE_TAG_FILENAME:
+    case GI_TYPE_TAG_UTF8:
+    case GI_TYPE_TAG_GLIST:
+    case GI_TYPE_TAG_GSLIST:
+        arg->v_pointer = NULL;
+        break;
+
+    case GI_TYPE_TAG_INTERFACE:
+        {
+            GIBaseInfo* interface_info;
+            GIInfoType interface_type;
+
+            interface_info = g_type_info_get_interface(type_info);
+            g_assert(interface_info != NULL);
+
+            interface_type = g_base_info_get_type(interface_info);
+
+            switch(interface_type) {
+            case GI_INFO_TYPE_ENUM:
+            case GI_INFO_TYPE_FLAGS:
+                arg->v_int = 0;
+                break;
+            case GI_INFO_TYPE_VALUE:
+                /* Better to use a non-NULL value holding NULL? */
+                arg->v_pointer = NULL;
+                break;
+            default:
+                arg->v_pointer = NULL;
+                break;
+            }
+
+            g_base_info_unref( (GIBaseInfo*) interface_info);
+        }
+        break;
+
+    case GI_TYPE_TAG_GHASH:
+        /* Possibly better to return an empty hash table? */
+        arg->v_pointer = NULL;
+        break;
+
+    case GI_TYPE_TAG_ARRAY:
+        arg->v_pointer = NULL;
+        break;
+
+    case GI_TYPE_TAG_SHORT:
+    case GI_TYPE_TAG_USHORT:
+    case GI_TYPE_TAG_INT:
+    case GI_TYPE_TAG_UINT:
+    case GI_TYPE_TAG_LONG:
+    case GI_TYPE_TAG_ULONG:
+    case GI_TYPE_TAG_SIZE:
+    case GI_TYPE_TAG_SSIZE:
+    case GI_TYPE_TAG_GTYPE:
+        /* these types are converted by normalize_int_types */
+        g_assert_not_reached();
+
+    default:
+        gjs_debug(GJS_DEBUG_ERROR,
+                  "Unhandled type %s for default GArgument initialization",
+                  g_type_tag_to_string(type_tag));
+        break;
+    }
+}
+
 JSBool
 gjs_value_to_arg(JSContext  *context,
                  jsval       value,
@@ -1389,19 +1529,14 @@ gjs_value_from_g_argument (JSContext  *context,
 
                 goto out;
             } else if (interface_type == GI_INFO_TYPE_FLAGS) {
-                /* This should be fixed to work without a GType, just like Enum */
                 gtype = g_registered_type_info_get_g_type((GIRegisteredTypeInfo*)interface_info);
-                if (gtype == G_TYPE_NONE) {
-                    gjs_throw(context,
-                              "Can't yet handle flags type '%s' that is not registered with GObject",
-                              g_base_info_get_name(interface_info));
-                    goto out;
-                }
-
                 if (_gjs_flags_value_is_valid(context, gtype, arg->v_int))
                     value = INT_TO_JSVAL(arg->v_int);
 
                 goto out;
+            } else if (interface_type == GI_INFO_TYPE_STRUCT &&
+                       g_struct_info_is_foreign((GIStructInfo*)interface_info)) {
+                return gjs_struct_foreign_convert_from_g_argument(context, value_p, type_info, arg);
             }
 
             /* Everything else is a pointer type, NULL is the easy case */
@@ -1635,7 +1770,7 @@ gjs_g_arg_release_internal(JSContext  *context,
                            GArgument  *arg)
 {
     JSBool failed;
-    
+
     g_assert(transfer != GI_TRANSFER_NOTHING);
 
     failed = JS_FALSE;
@@ -1680,6 +1815,11 @@ gjs_g_arg_release_internal(JSContext  *context,
 
             interface_type = g_base_info_get_type(interface_info);
 
+            if (interface_type == GI_INFO_TYPE_STRUCT &&
+                g_struct_info_is_foreign((GIStructInfo*)interface_info))
+                return gjs_struct_foreign_release_g_argument(context,
+                        transfer, type_info, arg);
+
             if (interface_type == GI_INFO_TYPE_ENUM || interface_type == GI_INFO_TYPE_FLAGS)
                 goto out;
 
