@@ -23,6 +23,8 @@
 #include <config.h>
 
 #include <gjs/gjs.h>
+#include <gi/foreign.h>
+
 #include <cairo.h>
 #include "cairo-private.h"
 
@@ -154,7 +156,7 @@ _GJS_CAIRO_CONTEXT_DEFINE_FUNC_BEGIN(method)                               \
                         #n1, &arg1, #n2, &arg2))                           \
         return JS_FALSE;                                                   \
     cr = gjs_cairo_context_get_context(context, obj);                      \
-    cfunc(cr, arg1, arg2);                                                 \
+    ret = cfunc(cr, arg1, arg2);                                           \
     *retval = BOOLEAN_TO_JSVAL(ret);                                       \
 _GJS_CAIRO_CONTEXT_DEFINE_FUNC_END
 
@@ -222,7 +224,6 @@ typedef struct {
 } GjsCairoContext;
 
 GJS_DEFINE_PROTO("CairoContext", gjs_cairo_context)
-
 GJS_DEFINE_PRIV_FROM_JS(GjsCairoContext, gjs_cairo_context_class);
 
 static void
@@ -374,6 +375,72 @@ _GJS_CAIRO_CONTEXT_DEFINE_FUNC2(translate, cairo_translate, "ff", double, tx, do
 _GJS_CAIRO_CONTEXT_DEFINE_FUNC0AFF(userToDevice, cairo_user_to_device)
 _GJS_CAIRO_CONTEXT_DEFINE_FUNC0AFF(userToDeviceDistance, cairo_user_to_device_distance)
 
+
+static JSBool
+appendPath_func(JSContext *context,
+                JSObject  *obj,
+                uintN      argc,
+                jsval     *argv,
+                jsval     *retval)
+{
+    JSObject *path_wrapper;
+    cairo_path_t *path;
+    cairo_t *cr;
+
+    if (!gjs_parse_args(context, "path", "o", argc, argv,
+                        "path", &path_wrapper))
+        return JS_FALSE;
+
+    path = gjs_cairo_path_get_path(context, path_wrapper);
+    if (!path) {
+        gjs_throw(context, "first argument to appendPath() should be a path");
+        return JS_FALSE;
+    }
+
+    cr = gjs_cairo_context_get_context(context, obj);
+    cairo_append_path(cr, path);
+    *retval = JSVAL_VOID;
+    return JS_TRUE;
+}
+
+static JSBool
+copyPath_func(JSContext *context,
+              JSObject  *obj,
+              uintN      argc,
+              jsval     *argv,
+              jsval     *retval)
+{
+    cairo_path_t *path;
+    cairo_t *cr;
+
+    if (!gjs_parse_args(context, "", "", argc, argv))
+        return JS_FALSE;
+
+    cr = gjs_cairo_context_get_context(context, obj);
+    path = cairo_copy_path(cr);
+    *retval = OBJECT_TO_JSVAL(gjs_cairo_path_from_path(context, path));
+    return JS_TRUE;
+}
+
+static JSBool
+copyPathFlat_func(JSContext *context,
+                  JSObject  *obj,
+                  uintN      argc,
+                  jsval     *argv,
+                  jsval     *retval)
+{
+    cairo_path_t *path;
+    cairo_t *cr;
+
+    if (!gjs_parse_args(context, "", "", argc, argv))
+        return JS_FALSE;
+
+    cr = gjs_cairo_context_get_context(context, obj);
+    path = cairo_copy_path_flat(cr);
+    *retval = OBJECT_TO_JSVAL(gjs_cairo_path_from_path(context, path));
+    return JS_TRUE;
+}
+
 static JSBool
 mask_func(JSContext *context,
           JSObject  *obj,
@@ -646,7 +713,7 @@ getTarget_func(JSContext *context,
     /* surface belongs to the context, so keep the reference */
     surface_wrapper = gjs_cairo_surface_from_surface(context, surface);
     if (!surface_wrapper) {
-        gjs_throw(context, "failed to create surface");
+        /* exception already set */
         return JS_FALSE;
     }
 
@@ -679,7 +746,7 @@ getGroupTarget_func(JSContext *context,
     /* surface belongs to the context, so keep the reference */
     surface_wrapper = gjs_cairo_surface_from_surface(context, surface);
     if (!surface_wrapper) {
-        gjs_throw(context, "failed to create surface");
+        /* exception already set */
         return JS_FALSE;
     }
 
@@ -689,7 +756,7 @@ getGroupTarget_func(JSContext *context,
 }
 
 static JSFunctionSpec gjs_cairo_context_proto_funcs[] = {
-    // appendPath
+    { "appendPath", appendPath_func, 0, 0},
     { "arc", arc_func, 0, 0 },
     { "arcNegative", arcNegative_func, 0, 0 },
     { "clip", clip_func, 0, 0 },
@@ -697,8 +764,8 @@ static JSFunctionSpec gjs_cairo_context_proto_funcs[] = {
     { "clipPreserve", clipPreserve_func, 0, 0 },
     { "closePath", closePath_func, 0, 0 },
     { "copyPage", copyPage_func, 0, 0 },
-    // copyPath
-    // copyPathFlat
+    { "copyPath", copyPath_func, 0, 0 },
+    { "copyPathFlat", copyPathFlat_func, 0, 0 },
     { "curveTo", curveTo_func, 0, 0 },
     { "deviceToUser", deviceToUser_func, 0, 0 },
     { "deviceToUserDistance", deviceToUserDistance_func, 0, 0 },
@@ -815,3 +882,65 @@ gjs_cairo_context_get_context(JSContext *context,
 
     return priv->cr;
 }
+
+static JSBool
+context_to_g_argument(JSContext      *context,
+                      jsval           value,
+                      GITypeInfo     *type_info,
+                      const char     *arg_name,
+                      GjsArgumentType argument_type,
+                      GITransfer      transfer,
+                      gboolean        may_be_null,
+                      GArgument      *arg)
+{
+    JSObject *obj;
+    cairo_t *cr;
+
+    obj = JSVAL_TO_OBJECT(value);
+    cr = gjs_cairo_context_get_context(context, obj);
+    if (!cr)
+        return JS_FALSE;
+    if (transfer == GI_TRANSFER_EVERYTHING)
+        cairo_destroy(cr);
+
+    arg->v_pointer = cr;
+    return JS_TRUE;
+}
+
+static JSBool
+context_from_g_argument(JSContext  *context,
+                        jsval      *value_p,
+                        GITypeInfo *type_info,
+                        GArgument  *arg)
+{
+    JSObject *obj;
+
+    obj = gjs_cairo_context_from_context(context, (cairo_t*)arg->v_pointer);
+    if (!obj)
+        return JS_FALSE;
+
+    *value_p = OBJECT_TO_JSVAL(obj);
+    return JS_TRUE;
+}
+
+static JSBool
+context_release_argument(JSContext  *context,
+                         GITransfer  transfer,
+                         GITypeInfo *type_info,
+                         GArgument  *arg)
+{
+    cairo_destroy((cairo_t*)arg->v_pointer);
+    return JS_TRUE;
+}
+
+static GjsForeignInfo foreign_info = {
+    context_to_g_argument,
+    context_from_g_argument,
+    context_release_argument
+};
+
+void
+gjs_cairo_context_init(JSContext *context)
+{
+    gjs_struct_foreign_register("cairo", "Context", &foreign_info);
+}
