@@ -46,10 +46,10 @@ function testLifeUniverseAndEverything() {
     assertEquals(-42, Everything.test_int64(-42));
 
     assertEquals(42, Everything.test_uint64(42));
-    
+
     assertEquals(42, Everything.test_short(42));
     assertEquals(-42, Everything.test_short(-42));
-    
+
     assertEquals(42, Everything.test_ushort(42));
 
     assertEquals(42, Everything.test_int(42));
@@ -116,9 +116,9 @@ function testNoImplicitConversionToUnsigned() {
 
 function testBadConstructor() {
     try {
-	Gio.AppLaunchContext();
+        Gio.AppLaunchContext();
     } catch (e) {
-	assert(e.message.indexOf("Constructor called as normal method") >= 0);
+        assert(e.message.indexOf("Constructor called as normal method") >= 0);
     }
 }
 
@@ -177,22 +177,29 @@ function testStaticMeth() {
 }
 
 function testClosure() {
+    let arguments_length = -1;
     let someCallback = function() {
+                           arguments_length = arguments.length;
                            return 42;
                        };
 
     let i = Everything.test_closure(someCallback);
 
+    assertEquals('callback arguments length', 0, arguments_length);
     assertEquals('callback return value', 42, i);
 }
 
 function testClosureOneArg() {
+    let arguments_length = -1;
     let someCallback = function(someValue) {
+                           arguments_length = arguments.length;
+                           assertEquals(1, arguments.length);
                            return someValue;
                        };
 
     let i = Everything.test_closure_one_arg(someCallback, 42);
 
+    assertEquals('callback arguments length', 1, arguments_length);
     assertEquals('callback with one arg return value', 42, i);
 }
 
@@ -203,7 +210,7 @@ function testCallback() {
     assertEquals('Callback', Everything.test_callback(callback), 42);
 
     assertEquals('CallbackNull', Everything.test_callback(null), 0);
-    assertEquals('CallbackUndefined', Everything.test_callback(undefined), 0);
+    assertRaises('CallbackUndefined', function () { Everything.test_callback(undefined) });
 }
 
 function testCallbackDestroyNotify() {
@@ -296,17 +303,17 @@ function testArrayOut() {
 // Convert an object to a predictable (not-hash-order-dependent) string
 function objToString(v) {
     if (typeof(v) == "object") {
-	let keys = [];
-	for (let k in v)
-	    keys.push(k);
-	keys.sort();
-	return "{" + keys.map(function(k) {
-	    return k + ":" + objToString(v[k]);
-	}) + "}";
+        let keys = [];
+        for (let k in v)
+            keys.push(k);
+        keys.sort();
+        return "{" + keys.map(function(k) {
+            return k + ":" + objToString(v[k]);
+        }) + "}";
     } else if (typeof(v) == "string") {
-	return '"' + v + '"';
+        return '"' + v + '"';
     } else {
-	return v;
+        return v;
     }
 }
 
